@@ -35,6 +35,7 @@
 #include <gdk/gdk.h>
 #include <gdk/gdkx.h>
 #include <stdlib.h>
+#include <pango/pango.h>
 
 #define GRADIENT_BAND_SIZE 4
 
@@ -636,30 +637,6 @@ eel_gdk_window_set_wm_protocols (GdkWindow *window,
 	g_free (atoms);
 }
 
-void
-eel_set_mini_icon (GdkWindow *window,
-			GdkPixmap *pixmap,
-			GdkBitmap *mask)
-{
-        GdkAtom icon_atom;
-        long data[2];
- 
-        g_return_if_fail (window != NULL);
-        g_return_if_fail (pixmap != NULL);
-
-        data[0] = GDK_WINDOW_XWINDOW (pixmap);
-        if (mask) {
-                data[1] = GDK_WINDOW_XWINDOW (mask);
-        } else {
-                data[1] = 0;
-	}
-
-        icon_atom = gdk_atom_intern ("KWM_WIN_ICON", FALSE);
-        gdk_property_change (window, icon_atom, icon_atom, 
-                             32, GDK_PROP_MODE_REPLACE,
-                             (guchar *) data, 2);
-}
-
 /**
  * eel_gdk_window_set_wm_hints_input:
  * 
@@ -755,6 +732,26 @@ eel_gdk_parse_geometry (const char *string, int *x_return, int *y_return,
 	return gdk_flags;
 }
 
+void
+eel_gdk_draw_layout_with_drop_shadow (GdkDrawable         *drawable,
+				      GdkGC               *gc,
+				      GdkColor            *text_color,
+				      GdkColor            *shadow_color,
+				      int                  x,
+				      int                  y,
+				      PangoLayout         *layout)
+{
+	gdk_draw_layout_with_colors (drawable, gc,
+				     x+1, y+1,
+				     layout,
+				     shadow_color, NULL);
+	
+	gdk_draw_layout_with_colors (drawable, gc,
+				     x, y,
+				     layout,
+				     text_color, NULL);
+}
+
 /* Color specifications have changed in gdk 2.0.  Unfortunately, nautilus has
    the old rgb:RRRR/BBBB/GGGG format scattered about in various xml files and
    the new gdk_color_parse () is not fully backward compatible. */
