@@ -40,14 +40,18 @@
 #include <gdk/gdkkeysyms.h>
 #include <gdk/gdkprivate.h>
 #include <gdk/gdkx.h>
+#include <gtk/gtkalignment.h>
 #include <gtk/gtkdialog.h>
 #include <gtk/gtkdnd.h>
+#include <gtk/gtkhbox.h>
+#include <gtk/gtkimage.h>
 #include <gtk/gtkmain.h>
 #include <gtk/gtkrc.h>
 #include <gtk/gtkselection.h>
 #include <gtk/gtkseparatormenuitem.h>
 #include <gtk/gtksignal.h>
 #include <pango/pangoft2.h>
+#include <math.h>
 
 #include "eel-marshal.h"
 #include "eel-marshal.c"
@@ -69,6 +73,37 @@
  */
 #define MAXIMUM_WM_REPARENTING_DEPTH 4
 
+/**
+ * eel_gtk_button_new_with_stock_icon:
+ * 
+ * @label: the button label, which may include a mnemonic accelerator.
+ * @stock_id: the stock icon to use.
+ *
+ * Creates a GtkButton with a mnemonic label and a stock icon.
+ *
+ * Return value: a new GtkButton widget.
+ **/
+GtkWidget*
+eel_gtk_button_new_with_stock_icon (const gchar *label, const gchar *stock_id)
+{
+	GtkWidget *button, *l, *image, *hbox, *align;
+
+	/* This is mainly copied from gtk_button_construct_child(). */
+	button = gtk_button_new ();
+	l = gtk_label_new_with_mnemonic (label);
+	gtk_label_set_mnemonic_widget (GTK_LABEL (l), GTK_WIDGET (button));
+	image = gtk_image_new_from_stock (stock_id, GTK_ICON_SIZE_BUTTON);
+	hbox = gtk_hbox_new (FALSE, 2);
+	align = gtk_alignment_new (0.5, 0.5, 0.0, 0.0);
+	gtk_box_pack_start (GTK_BOX (hbox), image, FALSE, FALSE, 0);
+	gtk_box_pack_end (GTK_BOX (hbox), l, FALSE, FALSE, 0);
+	gtk_container_add (GTK_CONTAINER (button), align);
+	gtk_container_add (GTK_CONTAINER (align), hbox);
+	gtk_widget_show_all (align);
+
+	return button;
+}
+
 /**
  * eel_gtk_button_set_padding
  * 
@@ -383,12 +418,14 @@ eel_gtk_window_set_initial_geometry (GtkWindow *window,
  * use this for the width.
  * @minimum_height: If the height from the string is smaller than this,
  * use this for the height.
+ * @ignore_position: If true position data from string will be ignored.
  */
 void
 eel_gtk_window_set_initial_geometry_from_string (GtkWindow *window, 
 						 const char *geometry_string,
 						 guint minimum_width,
-						 guint minimum_height)
+						 guint minimum_height,
+						 gboolean ignore_position)
 {
 	int left, top;
 	guint width, height;
@@ -414,47 +451,15 @@ eel_gtk_window_set_initial_geometry_from_string (GtkWindow *window,
 	if (geometry_flags & EEL_GDK_HEIGHT_VALUE) {
 		height = MAX (height, minimum_height);
 	}
+	
+	/* Ignore saved window position if requested. */
+	if (ignore_position) {
+		geometry_flags &= ~(EEL_GDK_X_VALUE | EEL_GDK_Y_VALUE);
+	}
 
 	eel_gtk_window_set_initial_geometry (window, geometry_flags, left, top, width, height);
 }
 
-/**
- * eel_gtk_signal_connect_free_data:
- * 
- * Function to displace the popup menu some, otherwise the first item
- * gets selected right away.
- * This function gets called by gtk_menu_popup ().
- *
- * @menu: the popup menu.
- * @x: x coord where gtk want to place the menu
- * @y: y coord where gtk want to place the menu
- * @user_data: something
- **/
-static void
-eel_popup_menu_position_func (GtkMenu  *menu,
-			      int      *x,
-			      int      *y,
-			      gboolean *push_in,
-			      gpointer  user_data)
-{
-	GdkPoint *offset;
-	GtkRequisition requisition;
-
-	g_assert (x != NULL);
-	g_assert (y != NULL);
-
-	offset = (GdkPoint *) user_data;
-
-	g_assert (offset != NULL);
-
-	gtk_widget_size_request (GTK_WIDGET (menu), &requisition);
-	  
-	*x = CLAMP (*x + (int) offset->x, 0, MAX (0, gdk_screen_width () - requisition.width));
-	*y = CLAMP (*y + (int) offset->y, 0, MAX (0, gdk_screen_height () - requisition.height));
-
-	*push_in = FALSE;
-}
-
 /**
  * eel_truncate_text_for_menu_item:
  * 
@@ -514,7 +519,7 @@ eel_pop_up_context_menu (GtkMenu	     *menu,
 	gtk_menu_popup (menu,					/* menu */
 			NULL,					/* parent_menu_shell */
 			NULL,					/* parent_menu_item */
-			eel_popup_menu_position_func,	/* func */
+			NULL,
 			&offset,			        /* data */
 			button,					/* button */
 			event ? event->time : GDK_CURRENT_TIME); /* activate_time */
@@ -944,6 +949,141 @@ eel_gtk_label_set_scale (GtkLabel *label,
 	pango_attr_list_unref (attr_list);
 }
 
+static void
+get_layout_location (GtkLabel  *label,
+                     gint      *xp,
+                     gint      *yp)
+{
+  GtkMisc *misc;
+  GtkWidget *widget;
+  float xalign;
+  int x, y;
+  int shadow_offset;
+  
+  shadow_offset = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (label),
+						      "eel-label-shadow-offset"));
+  
+  misc = GTK_MISC (label);
+  widget = GTK_WIDGET (label);
+  
+  if (gtk_widget_get_direction (widget) == GTK_TEXT_DIR_LTR)
+    xalign = misc->xalign;
+  else
+    xalign = 1.0 - misc->xalign;
+  
+  x = floor (widget->allocation.x + (int)misc->xpad
+             + ((widget->allocation.width - widget->requisition.width - shadow_offset) * xalign)
+             + 0.5);
+  
+  y = floor (widget->allocation.y + (int)misc->ypad 
+             + ((widget->allocation.height - widget->requisition.height - shadow_offset) * misc->yalign)
+             + 0.5);
+  
+
+  if (xp)
+    *xp = x;
+
+  if (yp)
+    *yp = y;
+}
+
+static gboolean
+eel_gtk_label_expose_event (GtkLabel *label, GdkEventExpose *event, gpointer user_data)
+{
+	int x, y;
+	GdkColor color;
+	GtkWidget *widget;
+	GdkGC *gc;
+	guint32 shadow_color;
+	int shadow_offset;
+	
+	shadow_color = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (label),
+							   "eel-label-shadow-color"));
+	shadow_offset = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (label),
+							    "eel-label-shadow-offset"));
+
+	color = eel_gdk_rgb_to_color (shadow_color);
+	
+	get_layout_location (label, &x, &y);
+
+	widget = GTK_WIDGET (label);
+	if (shadow_offset > 0) {
+		gc = gdk_gc_new (widget->window);
+		gdk_gc_set_rgb_fg_color (gc, &color);
+		gdk_gc_set_clip_rectangle (gc, &event->area);
+		
+		gdk_draw_layout (widget->window,
+				 gc,
+				 x + shadow_offset, y + shadow_offset,
+				 label->layout);
+		g_object_unref (gc);
+	}
+	
+	gtk_paint_layout (widget->style,
+			  widget->window,
+			  GTK_WIDGET_STATE (widget),
+			  FALSE,
+			  &event->area,
+			  widget,
+			  "label",
+			  x, y,
+			  label->layout);
+
+	return TRUE;
+}
+
+static void
+eel_gtk_label_size_request (GtkLabel *label, GtkRequisition *requisition, gpointer user_data)
+{
+	gint shadow_offset;
+
+	shadow_offset = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (label),
+							    "eel-label-shadow-offset"));
+
+	requisition->width += shadow_offset;
+	requisition->height += shadow_offset;
+}
+
+static void
+set_up_label (GtkLabel *label)
+{
+
+	if (g_object_get_data (G_OBJECT (label), "eel-label-set-up") != NULL) {
+		return;
+	}
+
+	g_signal_connect (label, "expose_event",
+			  G_CALLBACK (eel_gtk_label_expose_event), NULL);
+	g_signal_connect_after (label, "size_request",
+				G_CALLBACK (eel_gtk_label_size_request), NULL);
+
+	g_object_set_data (G_OBJECT (label), "eel-label-set-up", "eel-label-set-up");
+}
+
+void
+eel_gtk_label_set_drop_shadow_color (GtkLabel *label,
+				     guint32 color)
+{
+	set_up_label (label);
+
+	g_object_set_data (G_OBJECT (label), "eel-label-shadow-color",
+			   GINT_TO_POINTER (color));
+		
+	gtk_widget_queue_draw (GTK_WIDGET (label));
+}
+
+void
+eel_gtk_label_set_drop_shadow_offset (GtkLabel *label,
+				      gint offset)
+{
+	set_up_label (label);
+
+	g_object_set_data (G_OBJECT (label), "eel-label-shadow-offset",
+			   GINT_TO_POINTER (offset));
+	
+	gtk_widget_queue_draw (GTK_WIDGET (label));
+}
+
 void
 eel_gtk_widget_set_background_color (GtkWidget *widget,
 				     const char *color_spec)
@@ -1090,6 +1230,56 @@ eel_gtk_tree_view_cell_is_completely_visible (GtkTreeView          *tree_view,
 	return eel_gdk_rectangle_contains_rectangle (visible_rect, cell_rect);
 }
 
+static gboolean 
+tree_view_button_press_callback (GtkWidget *tree_view,
+				 GdkEventButton *event,
+				 gpointer data)
+{
+	GtkTreePath *path;
+	GtkTreeViewColumn *column;
+
+	if (event->button == 1 && event->type == GDK_BUTTON_PRESS) {
+		if (gtk_tree_view_get_path_at_pos (GTK_TREE_VIEW (tree_view),
+						   event->x, event->y,
+						   &path,
+						   &column,
+						   NULL, 
+						   NULL)) {
+			gtk_tree_view_row_activated
+				(GTK_TREE_VIEW (tree_view), path, column);
+		}
+	}
+
+	return FALSE;
+}
+
+void
+eel_gtk_tree_view_set_activate_on_single_click (GtkTreeView *tree_view,
+						gboolean should_activate)
+{
+	guint button_press_id;
+
+	button_press_id = GPOINTER_TO_UINT 
+		(g_object_get_data (G_OBJECT (tree_view), 
+				    "eel-tree-view-activate"));
+
+	if (button_press_id && !should_activate) {
+		g_signal_handler_disconnect (tree_view, button_press_id);
+		g_object_set_data (G_OBJECT (tree_view), 
+				   "eel-tree-view-activate", 
+				   0);
+	} else if (!button_press_id && should_activate) {
+		button_press_id = g_signal_connect 
+			(tree_view,
+			 "button_press_event",
+			 G_CALLBACK  (tree_view_button_press_callback),
+			 NULL);
+		g_object_set_data (G_OBJECT (tree_view), 
+				   "eel-tree-view-activate", 
+				   GUINT_TO_POINTER (button_press_id));
+	}
+}
+
 gboolean
 eel_gtk_viewport_get_visible_rect (GtkViewport  *viewport, 
 				   GdkRectangle *rect)
